from pyunity import *
import math

rt50 = math.sqrt(50)

class Rotator(Behaviour):
    def Update(self, dt):
        self.transform.localEulerAngles += Vector3(0, 45 * dt, 0)
        print(round(1 / dt, 3))

def main():
    mat = Material((255, 0, 0))

    scene = SceneManager.AddScene("Scene")
    scene.mainCamera.transform.localPosition = Vector3(0, 3, 0)
    scene.mainCamera.transform.localEulerAngles = Vector3(20, 0, 0)

    root = GameObject("Root")
    root.AddComponent(Rotator)
    scene.Add(root)

    cube = GameObject("Cube", root)
    cube.transform.localPosition = Vector3(0, 0, 10)
    renderer = cube.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.cube
    scene.Add(cube)

    quad = GameObject("Quad", root)
    quad.transform.localPosition = Vector3(rt50, 0, rt50)
    quad.transform.localEulerAngles = Vector3(0, -45, 0)
    renderer = quad.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.quad
    scene.Add(quad)

    sphere = GameObject("Sphere", root)
    sphere.transform.localPosition = Vector3(10, 0, 0)
    renderer = sphere.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.sphere
    scene.Add(sphere)

    capsule = GameObject("Capsule", root)
    capsule.transform.localPosition = Vector3(rt50, 0, -rt50)
    renderer = capsule.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.capsule
    scene.Add(capsule)

    cylinder = GameObject("Cylinder", root)
    cylinder.transform.localPosition = Vector3(0, 0, -10)
    renderer = cylinder.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.cylinder
    scene.Add(cylinder)

    sphere = GameObject("Sphere", root)
    sphere.transform.localPosition = Vector3(-rt50, 0, -rt50)
    renderer = sphere.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.sphere
    scene.Add(sphere)

    capsule = GameObject("Capsule", root)
    capsule.transform.localPosition = Vector3(-10, 0, 0)
    renderer = capsule.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.capsule
    scene.Add(capsule)

    quad = GameObject("Quad", root)
    quad.transform.localPosition = Vector3(-rt50, 0, rt50)
    quad.transform.localEulerAngles = Vector3(0, 45, 0)
    renderer = quad.AddComponent(MeshRenderer)
    renderer.mat = mat
    renderer.mesh = loader.Primitives.quad
    scene.Add(quad)
    
    SceneManager.LoadScene(scene)


"""
Version 0.2.0 (in development)
==============================

A Python implementation of the Unity Engine
that supports different types of windows.
Still in development.

Importing
---------
The first step in using PyUnity is always
importing it. A standard way to import is like
so:

    >>> from pyunity import *

Debug information is turned on by default. If
you want to turn it off, set the
PYUNITY_DEBUG_MODE environment variable to ``"0"``.
This is the output with debugging::

    Loaded config
    Trying FreeGLUT as a window provider
    FreeGLUT doesn't work, trying GLFW
    GLFW doesn't work, trying Pygame
    Using window provider Pygame
    Loaded PyUnity version 0.2.0

If debugging is off, there is no output:

    >>> import os
    >>> os.environ["PYUNITY_DEBUG_MODE"] = "0"
    >>> from pyunity import *
    >>> # No output

Scenes
------
All PyUnity projects start with a scene. To add
a scene, do this:

    >>> scene = SceneManager.AddScene("Scene 1")

Then, let's move the camera backwards 10 units.

    >>> scene.mainCamera.transform.position = Vector3(0, 0, -10)

Finally, add a cube at the origin:

    >>> cube = GameObject("Cube")
    >>> renderer = cube.AddComponent(MeshRenderer)
    >>> renderer.mesh = Mesh.cube(2)
    >>> renderer.mat = Material((255, 0, 0))
    >>> scene.Add(cube)

To see what you have added to the scene, call ``scene.List()``:

    >>> scene.List()
    /Main Camera
    /Light
    /Cube

Finally, to run the scene, call ``scene.Run()``. The window that
is created is one of FreeGLUT, GLFW or Pygame. The window is
selected on module initialization (see Windows subheading).

Behaviours
----------
To create your own PyUnity script, create a class that inherits
from Behaviour. Usually in Unity, you would put the class in its
own file, but Python can't do something like that, so put all of
your scripts in one file. Then, to add a script, just use
``AddComponent()``. Do not put anything in the ``__init__`` function,
instead put it in ``Start()``. The ``Update()`` function receives one
parameter, ``dt``, which is the same as ``Time.deltaTime``.

Windows
-------
The window is provided by one of three
providers: GLFW, Pygame and FreeGLUT.
When you first import PyUnity, it checks
to see if any of the three providers
work. The testing order is as above, so
Pygame is tested last.

To create your own provider, create a
class that has the following methods:

- ``__init__``: initiate your window and
  check to see if it works.
- ``start``: start the main loop in your
  window. The first parameter is
  ``update_func``, which is called
  when you want to do the OpenGL calls.

Check the source code of any of the window
providers for an example. If you have a
window provider, then please create a new
pull request.

Examples
--------
To run an example, import it like so:

    >>> from pyunity.examples.example1 import main
    Loaded config
    Trying FreeGLUT as a window provider
    FreeGLUT doesn't work, trying GLFW
    GLFW doesn't work, trying Pygame
    Using window provider Pygame
    Loaded PyUnity version 0.2.0
    >>> main()

Or from the command line:

    > python -m pyunity 1

The ``1`` just means to load example 1, and there
are 6 examples. To load all examples one by
one, do not specify a number. If you want to
contribute an example, then please create a
new pull request.

"""

__version__ = "0.2.0"
__copyright__ = "Copyright 2020 Ray Chen"
__email__ = "tankimarshal2@gmail.com"
__license__ = "MIT License"
__summary__ = "A Python implementation of the Unity Engine that supports different types of windows."
__title__ = "pyunity"
__uri__ = "https://pyunity.readthedocs.io/en/latest/"

import os
from .audio import *
from .core import *
from .vector3 import *
from .quaternion import *
from .scene import SceneManager
from .physics import *
from . import loader

def timer(func):
    """
    Use this decorator to print how long
    a function takes.

    """
    def wrapper(*args, **kwargs):
        start = time()
        func(*args, **kwargs)
        print(round(time() - start, 3))
    return wrapper

if os.environ["PYUNITY_DEBUG_MODE"] == "1":
    print("Loaded PyUnity version %s" % __version__)

if os.environ["PYUNITY_INTERACTIVE"] == "1": del window, gl
del core, scene, math, errors, operator, vector3, quaternion, physics, os


from . import (
    example1, example2, example3, example4, example5, example6, example7, example8
)
from ..scene import SceneManager

import sys

example_list = [
    example1,
    example2,
    example3,
    example4,
    example5,
    example6,
    example7,
    example8,
]

def show():
    if len(sys.argv) == 1: num = 0
    else: num = int(sys.argv[1])
    if not num:
        for index, example in enumerate(example_list):
            print("\nExample", index + 1)
            example.main()
            SceneManager.RemoveScene(SceneManager.GetSceneByName("Scene"))
    else:
        example_list[num - 1].main()


from pyunity import *

class Switch(Behaviour):
    a = 3
    def Update(self, dt):
        self.a -= dt
        if self.a < 0:
            SceneManager.LoadSceneByIndex(1)

def main():
    scene = SceneManager.AddScene("Scene")
    scene2 = SceneManager.AddScene("Scene 2")
    scene.mainCamera.transform.localPosition = Vector3(0, 0, -10)
    scene2.mainCamera.transform.localPosition = Vector3(0, 0, -10)

    cube = GameObject("Cube")
    renderer = cube.AddComponent(MeshRenderer)
    renderer.mesh = Mesh.cube(2)
    renderer.mat = Material((255, 0, 0))
    cube.AddComponent(Switch).scene2 = scene2
    scene.Add(cube)

    cube2 = GameObject("Cube 2")
    renderer = cube2.AddComponent(MeshRenderer)
    renderer.mesh = Mesh.cube(2)
    renderer.mat = Material((0, 0, 255))
    scene2.Add(cube2)

    SceneManager.LoadScene(scene)

if __name__ == "__main__":
    main()


from pyunity import *

def main():
    scene = SceneManager.AddScene("Scene")
    
    scene.mainCamera.transform.localPosition = Vector3(0, 3, -10)
    scene.mainCamera.transform.eulerAngles = Vector3(15, 0, 0)
    
    cube = GameObject("Cube")
    cube.transform.localPosition = Vector3(2, 0, 0)
    renderer = cube.AddComponent(MeshRenderer)
    renderer.mesh = Mesh.cube(2)
    renderer.mat = Material((255, 0, 0))
    collider = cube.AddComponent(AABBoxCollider)
    collider.SetSize(Vector3(1, -1, -1), Vector3(3, 1, 1))
    rb = cube.AddComponent(Rigidbody)
    rb.velocity = Vector3(-2, 0, 0)
    rb.gravity = False

    scene.Add(cube)
    
    cube = GameObject("Cube 2")
    cube.transform.localPosition = Vector3(5, 0, 0)
    renderer = cube.AddComponent(MeshRenderer)
    renderer.mesh = Mesh.cube(2)
    renderer.mat = Material((0, 0, 255))
    collider = cube.AddComponent(AABBoxCollider)
    collider.SetSize(Vector3(4, -1, -1), Vector3(6, 1, 1))
    rb = cube.AddComponent(Rigidbody)
    rb.velocity = Vector3(-4, 0, 0)
    rb.gravity = False

    scene.Add(cube)

    SceneManager.LoadScene(scene)

if __name__ == "__main__":
    main()


from pyunity import *
import os

class Rotator(Behaviour):
    def Update(self, dt):
        self.transform.eulerAngles += Vector3(0, 90, 0) * dt

def main():
    mesh = loader.LoadMesh(os.path.join(os.path.dirname(os.path.abspath(__file__)), "house.mesh"))

    scene = SceneManager.AddScene("Scene")

    scene.mainCamera.transform.localPosition = Vector3(0, 0, -20)

    house = GameObject("House")
    house.transform.eulerAngles = Vector3(0, 180, 0)
    renderer = house.AddComponent(MeshRenderer)
    renderer.mesh = mesh
    renderer.mat = Material((255, 0, 0))
    house.AddComponent(Rotator)
    
    scene.Add(house)
    
    SceneManager.LoadScene(scene)

if __name__ == "__main__":
    main()


from pyunity import *
from pyunity import config
import os

def main():
    scene = SceneManager.AddScene("Scene")
    if config.audio:
        path = os.path.realpath(os.path.dirname(__file__))
        clip = AudioClip(os.path.join(path, "explode.ogg"))
        scene.mainCamera.AddComponent(AudioSource).SetClip(clip)
    SceneManager.LoadScene(scene)

if __name__ == "__main__":
    main()


"""
pyunity.window
==============
A module used to load the window providers.

Windows
-------
The window is provided by one of three
providers: GLFW, Pygame and FreeGLUT.
When you first import PyUnity, it checks
to see if any of the three providers
work. The testing order is as above, so
Pygame is tested last.

To create your own provider, create a
class that has the following methods:

- `__init__`: initiate your window and
    check to see if it works.
- `start`: start the main loop in your
    window. The first parameter is
    ``update_func``, which is called
    when you want to do the OpenGL calls.

Check the source code of any of the window
providers for an example. If you have a
window provider, then please create a new
pull request.

"""

import os
import OpenGL.GLUT, glfw, pygame
glfw.ERROR_REPORTING = True
from ..errors import *

def glfwCheck():
    """Checks to see if GLFW works"""
    global glfw
    if not glfw.init():
        raise Exception
    glfw.create_window(5, 5, "a", None, None)
    glfw.terminate()
    del glfw

def pygameCheck():
    """Checks to see if Pygame works"""
    global pygame
    if pygame.init()[0] == 0:
        raise Exception
    del pygame

def glutCheck():
    """Checks to see if FreeGLUT works"""
    global OpenGL
    OpenGL.GLUT.glutInit()
    del OpenGL

def LoadWindowProvider():
    """Loads an appropriate window provider to use"""
    winfo = [
        ("GLFW", glfwCheck),
        ("Pygame", pygameCheck),
        ("FreeGLUT", glutCheck),
    ]

    windowProvider = ""
    success = True
    i = 0

    for name, checker in winfo:
        next = winfo[i + 1][0] if i < len(winfo) - 1 else None
        if os.environ["PYUNITY_DEBUG_MODE"] == "1":
            print("Trying", name, "as a window provider")
        try:
            checker()
            windowProvider = name
        except Exception as e:
            success = bool(next)
            if success and os.environ["PYUNITY_DEBUG_MODE"] == "1":
                print(name, "doesn't work, trying", next)
        
        if next is None: raise PyUnityException("No window provider found")
        if windowProvider: break
        i += 1

    if os.environ["PYUNITY_DEBUG_MODE"] == "1":
        print("Using window provider", windowProvider)

    if windowProvider == "FreeGLUT":
        from .glutWindow import Window as glutWindow
        return glutWindow
    if windowProvider == "GLFW":
        from .glfwWindow import Window as glfwWindow
        return glfwWindow
    if windowProvider == "Pygame":
        from .pygameWindow import Window as pygameWindow
        return pygameWindow



from setuptools import setup, find_packages, Extension
import os, glob, shutil, sys
if "cython" not in os.environ:
    os.environ["cython"] = "1"

with open("README.md", "r") as fh:
    long_description = fh.read()

if os.environ["cython"] == "1":
    c_files = glob.glob("src/**/*.c", recursive = True)
    mesh_files = glob.glob("src/**/*.mesh", recursive = True)
    config = {
        "package_dir": {"pyunity": "src"},
        "packages": ["pyunity"] + ["pyunity." + package for package in find_packages(where = "src")],
        "ext_package": "pyunity",
        "ext_modules": [Extension(file[4:-2].replace(os.path.sep, "."), [file]) for file in c_files],
        "package_data": {"pyunity": [file[4:] for file in mesh_files]},
    }
else:
    mesh_files = glob.glob("pyunity/**/*.mesh", recursive = True)
    config = {
        "packages": ["pyunity"] + ["pyunity." + package for package in find_packages(where = "pyunity")],
        "package_data": {"pyunity": [file[8:] for file in mesh_files]},
    }

setup(
    name = "pyunity",
    version = "0.2.0",
    author = "Ray Chen",
    author_email = "tankimarshal2@gmail.com",
    description = "A Python implementation of the Unity Engine",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github/rayzchen/PyUnity",
    classifiers = [
        "Development Status :: 3 - Alpha",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    setup_requires = [
        "cython",
    ],
    install_requires = [
        "glfw",
        "pygame",
        "pyopengl",
    ],
    python_requires = '>=3.6',
    **config,
)



from pyunity import *
import os

class Rotator(Behaviour):
    def Update(self, dt):
        self.transform.eulerAngles += Vector3(0, 90, 135) * dt

def main():
    mesh = loader.LoadMesh(os.path.join(os.path.dirname(os.path.realpath(__file__)), "cube.mesh"))

    scene = SceneManager.AddScene("Scene")

    scene.mainCamera.transform.localPosition = Vector3(0, 0, -10)

    cube = GameObject("Cube")
    cube.AddComponent(Rotator)
    renderer = cube.AddComponent(MeshRenderer)
    renderer.mesh = mesh
    renderer.mat = Material((255, 0, 0))
    scene.Add(cube)
    
    scene.List()
    SceneManager.LoadScene(scene)

if __name__ == "__main__":
    main()


"""
A basic 3D Physics engine that uses
similar concepts to the Unity
Engine itself. Only supports
non-rotated colliders.

To create an immoveable object, use
math.inf or the provided `infinity`
variable. This will make the object
not be able to move, unless you set
an initial velocity. Then, the
collider will either push everything
it collides with, or bounces it back
at twice the speed.

Example
-------

    >>> cube = GameObject("Cube")
    >>> collider = cube.AddComponent(AABBoxCollider)
    >>> collider.SetSize(-Vector3.one(), Vector3.one())
    >>> collider.velocity = Vector3.right()

Configuration
-------------
If you want to change some configurations, import
the config file like so:

    >>> from pyunity.physics import config

Inside the config file there are some configurations:

- `gravity` is the gravity of the whole system. It only
  affects Rigidbodies that have `gravity` set to True.

"""

from .core import *
del config